/*
** File: adb3.c  
** Project: ADB3 API library
** Purpose: OS-independent code for ADB3 API library
**
** (C) Copyright Alpha Data 2011
*/

#include <adb3.h>
#include <portability.h>
#include <adb3_platform.h>
#include "ioctl_adb3.h"
#if PORTABLE_HAS_WCHAR_SUPPORT
# include <portable_wstring.h>
#endif

#define ARRAY_LENGTH(x) (sizeof(x) / sizeof((x)[0]))

/* We might make this extern one day */
static ADB3_STATUS
ADB3_SetCooperativeLevel(
  ADB3_HANDLE hCard,
  uint32_t cooperativeLevel);

ADB3_EXPORT ADB3_STATUS
ADB3_CALLING_CONVENTION
ADB3_Close(
  ADB3_HANDLE hCard)
{
  /* This has to be done in an OS-specific way */
  return adb3CloseDevice(hCard);
}

ADB3_EXPORT ADB3_STATUS
ADB3_CALLING_CONVENTION
ADB3_GetProdTestInfo(
  ADB3_HANDLE hCard,
  ADB3_PRODTEST_INFO* pProdTestInfo)
{
  IOCTLS_ADB3_GETPRODTESTINFO ioctl;
  ADB3_STATUS status;

  if (NULL == pProdTestInfo) {
    return ADB3_NULL_POINTER;
  }

  status = adb3Ioctl(hCard, ADB3_IOCTLCODE_GETPRODTESTINFO, &ioctl, 0, sizeof(ioctl.out));
  if (ADB3_SUCCESS == status) {
    memset(pProdTestInfo, 0, sizeof(*pProdTestInfo));
    pProdTestInfo->SerialNumber = ioctl.out.serialNumber;
    pProdTestInfo->CPLDRevision = ioctl.out.cpldRevision;
    pProdTestInfo->PCBRevision = ioctl.out.pcbRevision;
    pProdTestInfo->BridgeRevision = ioctl.out.bridgeRevision;
    pProdTestInfo->BridgeDate = ioctl.out.bridgeDate;
    pProdTestInfo->BridgeTime = ioctl.out.bridgeTime;
  }
  return status;
}

ADB3_EXPORT const char*
ADB3_CALLING_CONVENTION
ADB3_GetStatusStringA(
  ADB3_STATUS code,
  bool_t bShort)
{
  static char buffer[128];

  if (bShort) {
    switch (code) {
    case ADB3_SUCCESS:
      return "ADB3_SUCCESS";

    case ADB3_INTERNAL_ERROR:
      return "ADB3_INTERNAL_ERROR";

    case ADB3_UNEXPECTED_ERROR:
      return "ADB3_UNEXPECTED_ERROR";

    case ADB3_BAD_DRIVER:
      return "ADB3_BAD_DRIVER";

    case ADB3_NO_MEMORY:
      return "ADB3_NO_MEMORY";

    case ADB3_ACCESS_DENIED:
      return "ADB3_ACCESS_DENIED";

    case ADB3_DEVICE_NOT_FOUND:
      return "ADB3_DEVICE_NOT_FOUND";

    case ADB3_HARDWARE_ERROR:
      return "ADB3_HARDWARE_ERROR";

    case ADB3_INVALID_BUFFER:
      return "ADB3_INVALID_BUFFER";

    case ADB3_INVALID_FLAG:
      return "ADB3_INVALID_FLAG";

    case ADB3_INVALID_HANDLE:
      return "ADB3_INVALID_HANDLE";

    case ADB3_INVALID_INDEX:
      return "ADB3_INVALID_INDEX";

    case ADB3_INVALID_REGION:
      return "ADB3_INVALID_REGION";

    case ADB3_NULL_POINTER:
      return "ADB3_NULL_POINTER";

    case ADB3_CANCELLED:
      return "ADB3_CANCELLED";

    case ADB3_RESOURCE_LIMIT:
      return "ADB3_RESOURCE_LIMIT";

    case ADB3_REGION_TOO_LARGE:
      return "ADB3_REGION_TOO_LARGE";

    case ADB3_DATATYPE_MISMATCH:
      return "ADB3_DATATYPE_MISMATCH";

    case ADB3_WRONG_MODE:
      return "ADB3_WRONG_MODE";

    case ADB3_NOT_SUPPORTED:
      return "ADB3_NOT_SUPPORTED";

    case ADB3_INVALID_PARAMETER:
      return "ADB3_INVALID_PARAMETER";

    default:
      break;
    }
  } else {
    switch (code) {
    case ADB3_SUCCESS:
      return "No error";

    case ADB3_INTERNAL_ERROR:
      return "An error in the API logic was detected";

    case ADB3_UNEXPECTED_ERROR:
      return "An unexpected error occurred";

    case ADB3_BAD_DRIVER:
      return "The driver may be incorrectly installed";

    case ADB3_NO_MEMORY:
      return "Could not allocate memory required to complete operation";

    case ADB3_ACCESS_DENIED:
      return "Caller does not have permission to perform the operation";

    case ADB3_DEVICE_NOT_FOUND:
      return "Failed to open the device with specified CardID";

    case ADB3_HARDWARE_ERROR:
      return "A hardware error occurred";

    case ADB3_INVALID_BUFFER:
      return "The supplied buffer was invalid";

    case ADB3_INVALID_FLAG:
      return "A flag was invalid or not recognized";

    case ADB3_INVALID_HANDLE:
      return "The device handle was invalid";

    case ADB3_INVALID_INDEX:
      return "The index parameter was invalid";

    case ADB3_INVALID_REGION:
      return "The offset and/or length parameter(s) were invalid";

    case ADB3_NULL_POINTER:
      return "A NULL pointer was passed where NULL was illegal";

    case ADB3_CANCELLED:
      return "The operation was cancelled";

    case ADB3_RESOURCE_LIMIT:
      return "A resource limit was reached";

    case ADB3_REGION_TOO_LARGE:
      return "The region is too large for a single operation";

    case ADB3_DATATYPE_MISMATCH:
      return "The datatype did not match that of the sensor";

    case ADB3_WRONG_MODE:
      return "The device was not in the proper mode";

    case ADB3_NOT_SUPPORTED:
      return "The operation is not supported for this device";

    case ADB3_INVALID_PARAMETER:
      return "A parameter had an illegal value";

    default:
      break;
    }
  }
  portableFormatStringA2(buffer, ARRAY_LENGTH(buffer), "Unknown status code %lu(0x%lx)", (unsigned long)code, (unsigned long)code);
  return buffer;
}

#if PORTABLE_HAS_WCHAR_SUPPORT
ADB3_EXPORT const wchar_t*
ADB3_CALLING_CONVENTION
ADB3_GetStatusStringW(
  ADB3_STATUS code,
  bool_t bShort)
{
  static const char* pBufferA;
  static wchar_t bufferW[128];

  pBufferA = ADB3_GetStatusStringA(code, bShort);
  portableUTF8ToWideChar(bufferW, ARRAY_LENGTH(bufferW), pBufferA);
  return bufferW;
}
#endif

ADB3_EXPORT ADB3_STATUS
ADB3_CALLING_CONVENTION
ADB3_GetUniqueId(
  ADB3_HANDLE hCard,
  ADB3_UNIQUE_ID* pUniqueId)
{
  IOCTLS_ADB3_GETUNIQUEID ioctl;
  ADB3_STATUS status;

  if (NULL == pUniqueId) {
    return ADB3_NULL_POINTER;
  }

  status = adb3Ioctl(hCard, ADB3_IOCTLCODE_GETUNIQUEID, &ioctl, 0, sizeof(ioctl.out));
  if (ADB3_SUCCESS == status) {
    memset(pUniqueId, 0, sizeof(*pUniqueId));
    pUniqueId->SerialNumber = ioctl.out.serialNumber;
    pUniqueId->Model = ioctl.out.model;
  }
  return status;
}

ADB3_EXPORT ADB3_STATUS
ADB3_CALLING_CONVENTION
ADB3_Open(
  unsigned int index,
  _ADB3_BOOL bPassive,
  _ADB3_UINT32 cooperativeLevel,
  ADB3_HANDLE* phCard)
{
  ADB3_STATUS status;
  ADB3_HANDLE hCard = ADB3_HANDLE_INVALID_VALUE;

  if (NULL == phCard) {
    return ADB3_NULL_POINTER;
  }

  /* This has to be done in an OS-specific way */
  status = adb3OpenDevice(index, bPassive, &hCard);
  if (ADB3_SUCCESS == status) {
    if (cooperativeLevel & ADB3_COOP_EXCLUSIVE) {
      status = ADB3_SetCooperativeLevel(hCard, cooperativeLevel);
      if (ADB3_SUCCESS == status) {
        *phCard = hCard;
      } else {
        ADB3_Close(hCard);
      }
    } else {
      *phCard = hCard;
    }
  }
  return status;
}

ADB3_EXPORT ADB3_STATUS
ADB3_CALLING_CONVENTION
ADB3_ServiceIRQ(
  ADB3_HANDLE hCard,
  _ADB3_BOOL* pbServiced)
{
  IOCTLS_ADB3_SERVICEIRQ ioctl;
  ADB3_STATUS status;

  status = adb3Ioctl(hCard, ADB3_IOCTLCODE_SERVICEIRQ, &ioctl, 0, sizeof(ioctl.out));
  if (ADB3_SUCCESS == status) {
    if (NULL != pbServiced) {
      *pbServiced = ioctl.out.bServicedIrq ? TRUE : FALSE;
    }
  }
  return status;
}

static ADB3_STATUS
ADB3_SetCooperativeLevel(
  ADB3_HANDLE hCard,
  uint32_t cooperativeLevel)
{
  IOCTLS_ADB3_SETCOOPLEVEL ioctl;

  ioctl.in.cooperativeLevel = cooperativeLevel;
  return adb3Ioctl(hCard, ADB3_IOCTLCODE_SETCOOPLEVEL, &ioctl, sizeof(ioctl.in), 0);
}
